@@ -35,7 +35,11 @@
 ;;; Code:
 
 (require 'mindstream-custom)
-(require 'magit-git)
+
+;; collisions unlikely within a single project repo
+;; and we don't want branch names to be too long as they
+;; may make things awkward in the Git UI.
+(defconst mindstream-branch-name-length 7)
 
 (defun mindstream--execute-shell-command (command &optional directory)
   "Execute shell COMMAND at DIRECTORY.
@@ -53,6 +57,22 @@ and arguments that are to be supplied to the command."
     (let ((exit-code (apply #'call-process cmd nil nil nil args)))
       exit-code)))
 
+(defun mindstream--execute-shell-command-to-string (command &optional directory)
+  "Execute COMMAND at DIRECTORY and return its stdout as a trimmed string.
+Returns nil if the command exits with a non-zero status."
+  (let ((cmd (car command))
+        (args (cdr command))
+        (default-directory (or directory
+                               (and (buffer-file-name)
+                                    (file-name-directory
+                                     (buffer-file-name)))
+                               default-directory)))
+    (with-temp-buffer
+      ;; 't' tells call-process to insert stdout into this temp buffer
+      (let ((exit-code (apply #'call-process cmd nil t nil args)))
+        (when (eq exit-code 0)
+          (string-trim (buffer-string)))))))
+
 (defun mindstream-backend-initialize (base-path)
   "Initialize the backend at the path BASE-PATH."
   (mindstream--execute-shell-command (list "git" "init")
@@ -76,12 +96,42 @@ and arguments that are to be supplied to the command."
 
 (defun mindstream-backend-root (&optional buffer)
   "Get the root folder of the VCS for BUFFER."
-  (let ((buffer (or buffer (current-buffer))))
-    (with-current-buffer buffer
-      ;; (vc-root-dir) returns nil in some cases,
-      ;; e.g. for an anonymous text session,
-      ;; but magit-toplevel seems to work.
-      (magit-toplevel))))
+  (with-current-buffer (or buffer (current-buffer))
+    (mindstream--execute-shell-command-to-string
+     (list "git" "rev-parse" "--show-toplevel"))))
+
+(defun mindstream-git-branch-name (&optional buffer)
+  "Get the Git branch name for BUFFER."
+  (with-current-buffer (or buffer (current-buffer))
+    (mindstream--execute-shell-command-to-string
+     (list "git" "branch" "--show-current"))))
+
+(defun mindstream--current-version (&optional buffer)
+  "Get the Git version hash for BUFFER."
+  (with-current-buffer (or buffer (current-buffer))
+    (mindstream--execute-shell-command-to-string
+     (list "git" "rev-parse" "HEAD"))))
+
+(defun mindstream-create-git-branch (&optional name)
+  "Start a new branch (stream) in the current repo.
+
+Note that this prefixes NAME with `mindstream-branch-prefix' before
+creating the branch."
+  (let ((branch-name (concat mindstream-branch-prefix
+                             "-"
+                             (or name
+                                 (mindstream--unique-name mindstream-branch-name-length))))
+        (start-point (mindstream--current-version)))
+    (mindstream--execute-shell-command
+     (list "git" "checkout" "-b" branch-name start-point))))
+
+(defun mindstream-backend-rename-branch (new-name)
+  "Rename the current branch to NEW-NAME.
+
+Note that this prefixes NAME with `mindstream-branch-prefix' before
+renaming the branch."
+  (let ((full-name (concat mindstream-branch-prefix "-" new-name)))
+    (mindstream--execute-shell-command (list "git" "branch" "-m" full-name))))
 
 (defun mindstream--session-dir (&optional buffer)
   "The repo base path containing BUFFER."
