@@ -36,40 +36,11 @@
 
 (require 'mindstream-custom)
 (require 'mindstream-backend)
+(require 'mindstream-stream)
 (require 'mindstream-util)
 
-(defvar mindstream-active-sessions nil
-  "A set of active sessions.
-
-Sessions are just paths to git repos.  A path being an \"active
-session\" means that any and all changes made at that path to
-git-tracked files will be versioned if they pull any
-`mindstream-triggers`.
-
-For now, this is implemented as a list for simplicity, since the
-number of active sessions is likely to be small.")
-
 (defvar mindstream-session-file-history nil)
 
-(defvar mindstream-session-history nil)
-
-(defun mindstream--unique-name ()
-  "Generate a unique name."
-  (sha1
-   (format "%s" (current-time))))
-
-(defun mindstream--session-file-name-relative (file dir)
-  "Return relative FILE name for `mindstream-session-history'.
-
-This returns the path of FILE relative to DIR if FILE is in DIR,
-otherwise it returns an abbreviated path (e.g. starting with ~
-if it is in the home folder)."
-  (if (string-match-p
-       (expand-file-name dir)
-       (expand-file-name file))
-      (file-relative-name file dir)
-    (abbreviate-file-name file)))
-
 (defun mindstream--starting-file-for-session (dir)
   "Select an appropriate starting file for a session.
 
@@ -87,41 +58,7 @@ filename relative to DIR rather than an absolute path."
                        nil t nil
                        mindstream-session-file-history))))
 
-(defun mindstream-begin-session ()
-  "Begin a session at the current path."
-  (interactive)
-  (push default-directory mindstream-active-sessions)
-  (add-to-list 'mindstream-session-history
-               (mindstream--session-file-name-relative default-directory
-                                                       mindstream-save-session-path))
-  (message "Session started at %s." default-directory))
-
-(defun mindstream-end-session (&optional session)
-  "End SESSION.
-
-This only removes implicit versioning.  It does not close any open
-buffers at the SESSION path."
-  (interactive (list (completing-read "Which session? "
-                                      mindstream-active-sessions
-                                      ;; We typically end sessions on existing
-                                      ;; projects to stop implicit versioning.
-                                      ;; Exclude anonymous sessions here since
-                                      ;; it's OK if those aren't ended (we can
-                                      ;; just forget about them and move on).
-                                      (lambda (session)
-                                        (not
-                                         (string-prefix-p mindstream-path
-                                                          session))))))
-  (let ((session (or session (mindstream--current-session))))
-    ;; session can be nil if called non-interactively
-    (setq mindstream-active-sessions
-          (remove session mindstream-active-sessions))
-    (message "Session %s ended." session)))
-
-(defun mindstream--current-session ()
-  "The session at the current path, if it is active."
-  (and (member default-directory mindstream-active-sessions)
-       default-directory))
+(define-obsolete-function-alias 'mindstream-begin-session 'mindstream-start-stream "2.0")
 
 (defun mindstream-native-session-p (&optional path)
   "Predicate to check whether PATH is a standard Mindstream path.
@@ -131,12 +68,6 @@ That is, either the anonymous session path or the archive path."
     (or (mindstream-anonymous-session-p path)
         (mindstream-archived-session-p path))))
 
-(defun mindstream-session-p (&optional path)
-  "Predicate to check whether PATH is an active session."
-  (let ((path (or path default-directory)))
-    (or (member path mindstream-active-sessions)
-        (mindstream-native-session-p path))))
-
 (defun mindstream--session-name ()
   "Name of the current session.
 
