@@ -34,6 +34,16 @@
 
 ;;; Code:
 
+(defun mindstream--unique-name (&optional truncate-to)
+  "Generate a unique name.
+
+This is a SHA1 hash.  Truncate the length to TRUNCATE-TO, if provided."
+  (let ((name (sha1
+               (format "%s" (current-time)))))
+    (if truncate-to
+        (seq-take name truncate-to)
+      name)))
+
 (defun mindstream--file-with-extension (extension path)
   "Return the first file with EXTENSION at PATH.
 
